/**
 * abssqr.com Inc.
 * Copyright (c) 2017-2020 All Rights Reserved.
 */

package com.abssqr.plat.core.service.ast;

import com.abssqr.plat.common.facade.enums.*;
import com.abssqr.plat.common.facade.model.rule.AstScreenRule;
import com.abssqr.plat.common.facade.param.ast.AstPackOprParam;
import com.abssqr.plat.common.model.convertor.web.AstPackParamConvert;
import com.abssqr.plat.common.model.domain.ast.AstPack;
import com.abssqr.plat.common.model.domain.task.CommonTask;
import com.abssqr.plat.common.model.repo.ast.AstPackRepo;
import com.abssqr.plat.common.model.repo.rule.AstScreenRuleRepo;
import com.abssqr.plat.common.model.repo.task.CommonTaskRepository;
import com.abssqr.plat.core.service.util.CommonTaskUtil;

import com.general.system.common.util.VarChecker;
import org.assertj.core.util.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author xiachenxiang
 * @version com.abssqr.plat.core.service.ast: AstPackManager.java, v 0.1 2020-04-18 4:16 PM xiachenxiang Exp $
 */
@Component
public class AstPackManager {
    @Autowired
    AstPackRepo          astPackRepository;
    @Autowired
    AstScreenRuleRepo    astScreenRuleRepo;
    @Autowired
    CommonTaskRepository commonTaskRepository;

    /**
     * 新增资产包
     *
     * @param param
     * @return
     */
    public String add(AstPackOprParam param) {
        // 1. 校验重复
        AstPack astPack = astPackRepository.getByAbsPackPackNo(param.getPackNo());
        VarChecker.checkArgument(astPack == null, "资产包已存在{0}", param.getPackNo());
        Long cnt = astPackRepository.cntByPackName(param.getPackName());
        VarChecker.checkEquals(cnt, 0L, "资产包名重复");
        // 2. 保存资产包
        astPack = AstPackParamConvert.param2Model(param);
        astPackRepository.insert(astPack);

        // 3. 保存筛选规则
        astScreenRuleRepo.insertBatch(astPack.getScreenRuleList());

        // 4. 插入资产筛选任务
        CommonTask commonTask = CommonTaskUtil.buildCommonTaskWithContext(param.getPackNo(),CommonTaskTypeEnum.PACK_MATCH,param.getAstPoolNo());
        commonTaskRepository.create(commonTask);
        return param.getPackNo();
    }

    /**
     * 创建产品默认的资产池
     * @param astPoolNo
     * @param astTypeEnum
     */
    public void craeteDefaultPack(String astPoolNo, AstTypeEnum astTypeEnum,String prodNo) {
        AstPack astPack = new AstPack();
        astPack.setAstPoolNo(astPoolNo);
        astPack.setPackNo(astPoolNo);
        astPack.setPackType(AstPackTypeEnum.DEFAULT);
        astPack.setStatus(AstPackStatusEnum.INIT);
        astPack.setAstType(astTypeEnum);
        createDefaultScreenRule(prodNo,astPack.getPackNo());
        astPackRepository.insert(astPack);
    }

    /**
     * 创建产品默认的筛选规则（去数据仓库筛选产品编码对应的的资产）
     * @param prodNo
     * @param packNo
     */
    //TODO 加筛选规则
    public void createDefaultScreenRule(String  prodNo,String packNo){
        AstScreenRule astScreenRule=new AstScreenRule();
        astScreenRule.setRelNo(packNo);
        astScreenRule.setRelType(AstScreenRelTypeEnum.PACK);
        astScreenRule.setRightCondition(AstScRuleRightCondEnum.eq);
        astScreenRule.setRightValue(prodNo);
        astScreenRuleRepo.insertBatch(Lists.newArrayList(astScreenRule));
    }
}
